/*
 * Copyright (C) 2000  Florian Sander
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#define SEEN_JOIN 1
#define SEEN_PART 2
#define SEEN_SIGN 3
#define SEEN_NICK 4
#define SEEN_NCKF 5
#define SEEN_KICK 6
#define SEEN_SPLT 7
#define SEEN_REJN 8
#define SEEN_CHPT 9
#define SEEN_CHJN 10

typedef struct gseen_data {
  struct gseen_data *next;
  int type;
  char *nick;
  char *host;
  char *chan;
  char *msg;
  time_t when;
  int spent;
} seendat;

typedef struct gseen_result {
  struct gseen_result *next;
  seendat *seen;
} gseenres;

typedef struct gseen_requests {
  struct gseen_requests *next;
  char *who;
  char *host;
  char *chan;
  time_t when;
} seenreq_by;

typedef struct gseen_request {
  struct gseen_request *next;
  char *nick;
  struct gseen_requests *by;
} seenreq;

typedef struct gseen_ignorewords {
  struct gseen_ignorewords *next;
  char *word;
} ignoredword;

static int gseen_expmem();
static void free_gseen();
static void add_seen(int, char *, char *, char *, char *, time_t, int);
static int get_spent(char *, char *);
static void write_seens();
static void read_seens();
static char *do_seen(char *, char *, char *, char *, int);
static seendat *findseen(char *);
static void add_seenresult(seendat *);
static int expmem_seenresults();
static int count_seenresults();
static void free_seenresults();
static void sortresults();
static char *do_seennick(seendat *);
static char *duration(int);
static int onchan(char *, char *);
static int count_seens();
static char *do_seenstats();
static void add_seenreq(char *, char *, char *, char *, time_t);
static int expmem_seenreq();
static void free_seenreq();
static void sortrequests(seenreq *);
static void report_seenreq(char *);
static int count_seenreq(seenreq_by *b);
static int expmem_ignoredwords();
static void free_ignoredwords();
static void add_ignoredword(char *word);
static int word_is_ignored(char *word);
static void purge_seens();
static int seenflood();
static int secretchan(char *);
static int nopub(char *);
static int quietseen(char *);
static int quietaiseens(char *);
static int nolog(char *);
